-- ======================= create_ip_forward_entry.adb =======================
-- 
--  Copyright 2001 by Craig Spannring
-- 
--  Def:   Create a route in the local computer's IP routing table
-- 
-- ===========================================================================


with Ada.Text_IO; use Ada.Text_IO;
with Ada.Command_Line;
with Ada.Exceptions;


with Iphlpapi.thin;    use Iphlpapi.Thin; 
with Iphlpapi.Helpers; use Iphlpapi.Helpers; 

procedure Create_Ip_Forward_Entry is 
   rcs_body_revision_id: constant string := "$Header: /home/cts/PersonalRepository/iphlpapi/Attic/create_ip_forward_entry.adb,v 1.1.2.3 2001/03/24 04:32:51 cts Exp $";
   
   package Cl renames Ada.Command_Line;
   
   use type DWORD;
   
   procedure Usage is 
   begin
      Put_Line("Usage- " & Cl.Command_Name 
               & " dest mask gateway [interface_index]");
      New_Line;
      Put_Line("   dest, mask and gateway must a decimal dotted quad format");
      Put_Line("   interface is the index of the interface you wish to change");
   end Usage;
   
   
   Rc:        Dword;
   Route:     aliased MIB_IPFORWARDROW;
   dest:      Ipaddr; 
   Mask:      Ipaddr;
   gateway:   Ipaddr;
   Interface: DWORD   := 2;
begin
   if Cl.Argument_Count < 3 or Cl.argument_Count > 4 then 
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
      return;
   end if;
   Dest    := String_To_Ip_Address(Cl.Argument(1));
   Mask    := String_To_Ip_Address(Cl.Argument(2));
   Gateway := String_To_Ip_Address(Cl.Argument(3));
   if Cl.Argument_Count >= 4 then 
      Interface := DWORD'Value(Cl.Argument(4));
   end if;
   
   Route := (dwForwardDest       => dest,
             dwForwardMask       => mask,
             dwForwardPolicy     => 0,
             dwForwardNextHop    => gateway,
             dwForwardIfIndex    => interface,
             dwForwardType       => 4,
             dwForwardProto      => PROTO_IP_NETMGMT,
             dwForwardAge        => 0,
             dwForwardNextHopAS  => 0,
             dwForwardMetric1    => 1,
             dwForwardMetric2    => 1,
             dwForwardMetric3    => 1,
             dwForwardMetric4    => 1,
             dwForwardMetric5    => 1);
             
   Rc := CreateIpForwardEntry(Route'Unchecked_Access);
   
   if Rc /= 0 then 
      Cl.Set_Exit_Status(Cl.Failure);
      Put_Line("Error.  rc = " & DWORD'Image(Rc));
   end if;
exception
   when Exc: others => 
      Put_Line(Ada.Exceptions.Exception_Message(Exc));
      Usage;
      Cl.Set_Exit_Status(Cl.Failure);
end Create_Ip_Forward_Entry;
